<?php

/**
 * OpenKM, Open Document Management System (http://www.openkm.com)
 * Copyright (c) 2006-2014 Paco Avila & Josep Llort
 * 
 * No bytes were intentionally harmed during the development of this application.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

namespace openkm\impl;

use openkm\bean\Folder;
use openkm\bean\Document;
use openkm\bean\Note;
use openkm\bean\Version;
use openkm\bean\LockInfo;
use openkm\bean\Mail;
use openkm\definition\BaseMail;
use openkm\util\UriHelper;
use openkm\exception\PathNotFoundException;
use Httpful\Request;
use Httpful\Mime;

/**
 * MailImpl
 *
 * @author sochoa
 */
class MailImpl implements BaseMail {

    private $host;
    private $user;
    private $password;

    /**
     * Construct
     */
    public function __construct($host, $user, $password) {
        $this->host = $host;
        $this->user = $user;
        $this->password = $password;
    }

    /**
     * getClient
     */
    private function getClient(Request $client) {
        $client->sendsXml();
        $client->authenticateWith($this->user, $this->password);
        return $client->send();
    }

    /**
     * getClientWithHTMLResponse
     */
    private function getClientWithHTMLResponse(Request $client) {
        $client->sendsXml();
        $client->expectsHtml();
        $client->authenticateWith($this->user, $this->password);
        return $client->send();
    }

    public function createMail(Mail $mail) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_CREATE);
        $client = Request::post($uri);
        $mailXML = new \SimpleXMLElement('<mail></mail>');
        $mailXML->addChild('author', $mail->getAuthor());
        $mailXML->addChild('created', $mail->getCreated());
        $mailXML->addChild('nodeClass', $mail->getNodeClass());
        $mailXML->addChild('path', $mail->getPath());
        $mailXML->addChild('permissions', $mail->getPermissions());
        $mailXML->addChild('subscribed', $mail->isSubscribed());
        $mailXML->addChild('uuid', $mail->getUuid());

        $mailXML->addChild('sentDate', $mail->getSentDate());
        $mailXML->addChild('receivedDate', $mail->getReceivedDate());
        $mailXML->addChild('from', $mail->getFrom());
        $mailXML->addChild('subject', $mail->getSubject());
        $mailXML->addChild('content', $mail->getContent());
        $mailXML->addChild('mimeType', $mail->getMimeType());
        $mailXML->addChild('headers', $mail->getHeaders());
        $mailXML->addChild('raw', $mail->getRaw());
        $mailXML->addChild('origin', $mail->getOrigin());
        $mailXML->addChild('size', $mail->getSize());
        if ($mail->istHasAttachments() == 1) {
            $mailXML->addChild('hasAttachments', 'true');
        } else {
            $mailXML->addChild('hasAttachments', 'false');
        }
        foreach ($mail->getReply() as $reply) {
            $mailXML->addChild('reply', $reply);
        }
        foreach ($mail->getTo() as $to) {
            $mailXML->addChild('to', $to);
        }
        foreach ($mail->getCc() as $cc) {
            $mailXML->addChild('cc', $cc);
        }
        foreach ($mail->getBcc() as $bcc) {
            $mailXML->addChild('bcc', $bcc);
        }
        foreach ($mail->getCategories() as $category) {
            $categoryXML = $mailXML->addChild('categories');
            $categoryXML->addChild('author', $category->getAuthor());
            $categoryXML->addChild('created', $category->getCreated());
            $categoryXML->addChild('nodeClass', $category->getNodeClass());
            $categoryXML->addChild('path', $category->getPath());
            $categoryXML->addChild('permissions', $category->getPermissions());
            $categoryXML->addChild('subscribed', $category->isSubscribed());
            $categoryXML->addChild('uuid', $category->getUuid());
            $categoryXML->addChild('hasChildren', $category->isHasChildren());
            $categoryXML->addChild('style', $category->getStyle());
        }
        foreach ($mail->getKeywords() as $keyword) {
            $mailXML->addChild('keywords', $keyword);
        }

        foreach ($mail->getNotes() as $note) {
            $noteXML = $mailXML->addChild('notes');
            $noteXML->addChild('author', $note->getAuthor());
            $noteXML->addChild('date', $note->getDate());
            $noteXML->addChild('path', $note->getPath());
            $noteXML->addChild('text', $note->getText());
        }
        $client->body($mailXML->asXML());
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return $this->phpMail($response->body);
    }

    public function getMailProperties($mailId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_GET_PROPERTIES);
        $uri .= '?mailId=' . $mailId;
        $client = Request::get($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return $this->phpMail($response->body);
    }

    public function deleteMail($mailId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_DELETE);
        $uri .= '?mailId=' . $mailId;
        $client = Request::delete($uri);
        $this->getClient($client);
    }

    public function renameMail($mailId, $newName) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_RENAME);
        $uri .= '?mailId=' . $mailId . '&newName=' . $newName;
        $client = Request::put($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return $this->phpMail($response->body);
    }

    public function moveMail($mailId, $dstId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_MOVE);
        $uri .= '?mailId=' . $mailId . '&dstId=' . $dstId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function getMailChildren($fldId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_GET_CHILDREN);
        $uri .= '?fldId=' . $fldId;
        $client = Request::get($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        $mails = array();
        foreach ($response->body->mail as $mailXML) {
            $mails[] = $this->phpMail($mailXML);
        }
        return $mails;
    }

    public function isValidMail($mailId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_IS_VALID);
        $uri .= '?mailId=' . $mailId;
        $client = Request::get($uri);
        $response = $this->getClientWithHTMLResponse($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return (string) $response->body;
    }

    public function getMailPath($uuid) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_GET_PATH);
        $uri .= '/' . $uuid;
        $client = Request::put($uri);
        $response = $this->getClientWithHTMLResponse($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return (string) $response->body;
    }

    public function createAttachment($mailId, $docName, $content) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_CREATE_ATTACHMENTS);
        $client = Request::post($uri);
        $client->body('<xml><docPath>' . $docPath . '</docPath><content>' . $is . '</content></xml>');
        $response = $this->getClient($client);
        return $this->phpDocument($response->body);
    }

    public function deleteAttachment($mailId, $docId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_DELETE_ATTACHMENTS);
        $uri .= '?mailId=' . $mailId . '&docId=' . $docId;
        $client = Request::delete($uri);
        $this->getClient($client);
    }

    public function getAttachments($mailId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_GET_ATTACHMENTS);
        $uri .= '?mailId=' . $mailId;
        $client = Request::get($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        $documents = array();
        foreach ($response->body->document as $documentXML) {
            $documents[] = $this->phpDocument($documentXML);
        }
        return $documents;
    }

    public function purgeMail($mailId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_PURGE);
        $uri .= '?mailId=' . $mailId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function copyMail($mailId, $dstId) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_COPY);
        $uri .= '?mailId=' . $mailId . '&dstId=' . $dstId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function extendedMailCopy($mailId, $dstId, $categories, $keywords, $propertyGroups, $notes, $wiki) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_EXTENDED_COPY);
        $uri .= '?mailId=' . $mailId . '&dstId=' . $dstId;
        if ($categories == 1) {
            $uri .= '&categories=true';
        } else {
            $uri.= '&categories=false';
        }
        if ($keywords == 1) {
            $uri .= '&keywords=true';
        } else {
            $uri.= '&keywords=false';
        }
        if ($propertyGroups == 1) {
            $uri .= '&propertyGroups=true';
        } else {
            $uri .= '&propertyGroups=false';
        }
        if ($notes == 1) {
            $uri .= '&notes=true';
        } else {
            $uri.= '&notes=false';
        }
        if ($wiki == 1) {
            $uri .= '&wiki=true';
        } else {
            $uri.= '&wiki=false';
        }
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function sendMailWithAttachments($subject, $body, $dstId, $recipients = array(), $docsId = array()) {
        $uri = UriHelper::getUri($this->host, UriHelper::MAIL_SEND_MAIL_WITH_ATTACHMENTS);
        $params['subject'] = $subject;
        $params['dstId'] = $dstId;
        $mails = '';
        foreach ($recipients as $value) {
            $mails .= $value . ';';
        }

        $params['recipients'] = substr($mails, 0, strlen($mails) - 1);
        $documents = '';
        foreach ($docsId as $value) {
            $documents .= $value . ';';
        }
        $params['docsId'] = substr($documents, 0, strlen($documents) - 1);
        $params['body'] = $body;
        $client = Request::post($uri);
        $client->body($params, Mime::UPLOAD);
        $client->authenticateWith($this->user, $this->password);
        $client->send();
    }

    public function phpMail($mailXML) {
        $mail = new Mail();
        $mail->setAuthor((string) $mailXML->author);
        $mail->setCreated((string) $mailXML->created);
        $mail->setNodeClass((int) $mailXML->nodeClass);
        $mail->setPath((string) $mailXML->path);
        $mail->setPermissions((int) $mailXML->permissions);
        $mail->setSubscribed((string) $mailXML->subscribed);
        $mail->setUuid((string) $mailXML->uuid);

        $mail->setSentDate((string) $mailXML->sentDate);
        $mail->setReceivedDate((string) $mailXML->receivedDate);
        $mail->setFrom((string) $mailXML->from);
        $mail->setSubject((string) $mailXML->subject);
        $mail->setContent((string) $mailXML->content);
        $mail->setMimeType((string) $mailXML->mimeType);
        $mail->setHeaders((string) $mailXML->headers);
        $mail->setRaw((string) $mailXML->raw);
        $mail->setOrigin((string) $mailXML->origin);
        $mail->setSize((int) $mailXML->size);
        $mail->setHasAttachments((string) $mailXML->hasAttachments);
        //reply
        $replyArray = array();
        foreach ($mailXML->reply as $reply) {
            $replyArray[] = $reply;
        }
        $mail->setReply($replyArray);
        //to
        $toArray = array();
        foreach ($mailXML->to as $to) {
            $toArray[] = $to;
        }
        $mail->setTo($toArray);
        //cc
        $ccArray = array();
        foreach ($mailXML->cc as $cc) {
            $ccArray[] = $cc;
        }
        $mail->setCc($ccArray);
        //bcc
        $bccArray = array();
        foreach ($mailXML->bcc as $bcc) {
            $bccArray[] = $bcc;
        }
        $mail->setBcc($bccArray);
        //categories
        $categories = array();
        foreach ($mailXML->categories as $categoryXML) {
            $categories[] = $this->phpFolder($categoryXML);
        }
        $mail->setCategories($categories);
        //keywords
        $keywords = array();
        foreach ($mailXML->keywords as $keywordXML) {
            $keywords[] = (string) $keywordXML;
        }
        $mail->setKeywords($keywords);
        //notes
        $notes = array();
        foreach ($mailXML->notes as $noteXML) {
            $note = new Note();
            $note->setAuthor((string) $noteXML->author);
            $note->setDate((string) $noteXML->date);
            $note->setPath((string) $noteXML->path);
            $note->setText((string) $noteXML->text);
            $notes[] = $note;
        }
        $mail->setNotes($notes);
        return $mail;
    }

    public function phpDocument($documentXML) {
        $document = new Document();
        $document->setAuthor((string) $documentXML->author);
        $document->setCreated((string) $documentXML->created);
        $document->setNodeClass((int) $documentXML->nodeClass);
        $document->setPath((string) $documentXML->path);
        $document->setPermissions((int) $documentXML->permissions);
        $document->setSubscribed((string) $documentXML->subscribed);
        $document->setUuid((string) $documentXML->uuid);

        //Version        
        $document->setActualVersion($this->phpVersion($documentXML->actualVersion));

        $document->setCheckedOut((string) $documentXML->checkedOut);
        $document->setConvertibleToDxf((string) $documentXML->convertibleToDxf);
        $document->setConvertibleToPdf((string) $documentXML->convertibleToPdf);
        $document->setConvertibleToSwf((string) $documentXML->convertibleToSwf);
        $document->setDescription((string) $documentXML->description);
        $document->setLanguage((string) $documentXML->language);
        $document->setLastModified((string) $documentXML->lastModified);

        //LockInfo        
        $document->setLockInfo($this->phpLockInfo($documentXML->lockInfo));

        $document->setLocked((string) $documentXML->locked);
        $document->setMimeType((string) $documentXML->mimeType);
        $document->setSigned((string) $documentXML->signed);
        $document->setTitle((string) $documentXML->title);

        //categories
        $categories = array();
        foreach ($documentXML->categories as $categoryXML) {
            $categories[] = $this->phpFolder($categoryXML);
        }
        $document->setCategories($categories);
        //keywords
        $keywords = array();
        foreach ($documentXML->keywords as $keywordXML) {
            $keywords[] = (string) $keywordXML;
        }
        $document->setKeywords($keywords);
        //notes
        $notes = array();
        foreach ($documentXML->notes as $noteXML) {
            $note = new Note();
            $note->setAuthor((string) $noteXML->author);
            $note->setDate((string) $noteXML->date);
            $note->setPath((string) $noteXML->path);
            $note->setText((string) $noteXML->text);
            $notes[] = $note;
        }
        $document->setNotes($notes);
        $subscriptors = array();
        foreach ($documentXML->subscriptors as $subscriptor) {
            $subscriptors[] = $subscriptor;
        }
        $document->setSubscriptors($subscriptors);
        return $document;
    }

    public function phpVersion($versionXML) {
        $version = new Version();
        $version->setActual((string) $versionXML->actual);
        $version->setAuthor((string) $versionXML->author);
        $version->setChecksum((string) $versionXML->checksum);
        $version->setCreated((string) $versionXML->created);
        $version->setName((string) $versionXML->name);
        $version->setSize((string) $versionXML->size);
        return $version;
    }

    public function phpLockInfo($lockInfoXML) {
        $lockInfo = new LockInfo();
        $lockInfo->setNodePath((string) $lockInfoXML->nodePath);
        $lockInfo->setOwner((string) $lockInfoXML->owner);
        $lockInfo->setToken((string) $lockInfoXML->token);
        return $lockInfo;
    }

    public function phpFolder($folderXML) {
        $folder = new Folder();
        $folder->setAuthor((string) $folderXML->author);
        $folder->setCreated((string) $folderXML->created);
        $folder->setNodeClass((int) $folderXML->nodeClass);
        $folder->setPath((string) $folderXML->path);
        $folder->setPermissions((int) $folderXML->permissions);
        $folder->setSubscribed((string) $folderXML->subscribed);
        $folder->setUuid((string) $folderXML->uuid);
        $folder->setHasChildren((string) $folderXML->hasChildren);
        $folder->setStyle((int) $folderXML->style);
        return $folder;
    }

}
