<?php

/**
 * OpenKM, Open Document Management System (http://www.openkm.com)
 * Copyright (c) 2006-2014 Paco Avila & Josep Llort
 * 
 * No bytes were intentionally harmed during the development of this application.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

namespace openkm\impl;

use openkm\bean\Folder;
use openkm\bean\Note;
use openkm\bean\LockInfo;
use openkm\bean\Record;
use openkm\definition\BaseRecord;
use openkm\util\UriHelper;
use openkm\exception\PathNotFoundException;
use Httpful\Request;

/**
 * RecordImpl
 *
 * @author sochoa
 */
class RecordImpl implements BaseRecord {

    private $host;
    private $user;
    private $password;

    /**
     * Construct
     */
    public function __construct($host, $user, $password) {
        $this->host = $host;
        $this->user = $user;
        $this->password = $password;
    }

    /**
     * getClient
     */
    private function getClient(Request $client) {
        $client->sendsXml();
        $client->authenticateWith($this->user, $this->password);
        return $client->send();
    }

    /**
     * getClientWithHTMLResponse
     */
    private function getClientWithHTMLResponse(Request $client) {
        $client->sendsXml();
        $client->expectsHtml();
        $client->authenticateWith($this->user, $this->password);
        return $client->send();
    }

    public function createRecord(Record $record) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_CREATE);
        $client = Request::post($uri);
        $recordXML = new \SimpleXMLElement('<record></record>');
        $recordXML->addChild('author', $record->getAuthor());
        $recordXML->addChild('created', $record->getCreated());
        $recordXML->addChild('nodeClass', $record->getNodeClass());
        $recordXML->addChild('path', $record->getPath());
        $recordXML->addChild('permissions', $record->getPermissions());
        $recordXML->addChild('subscribed', $record->isSubscribed());
        $recordXML->addChild('uuid', $record->getUuid());
        if ($record->getLockInfo() != null) {
            //LockInfo
            $recordXML->addChild('nodePath', $record->getLockInfo()->getNodePath());
            $recordXML->addChild('owner', $record->getLockInfo()->getOwner());
            $recordXML->addChild('token', $record->getLockInfo()->getToken());
        }
        $recordXML->addChild('locked', $record->isLocked());
        $recordXML->addChild('title', $record->getTitle());

        foreach ($record->getCategories() as $category) {
            $categoryXML = $recordXML->addChild('categories');
            $categoryXML->addChild('author', $category->getAuthor());
            $categoryXML->addChild('created', $category->getCreated());
            $categoryXML->addChild('nodeClass', $category->getNodeClass());
            $categoryXML->addChild('path', $category->getPath());
            $categoryXML->addChild('permissions', $category->getPermissions());
            $categoryXML->addChild('subscribed', $category->isSubscribed());
            $categoryXML->addChild('uuid', $category->getUuid());
            $categoryXML->addChild('hasChildren', $category->isHasChildren());
            $categoryXML->addChild('style', $category->getStyle());
        }
        foreach ($record->getKeywords() as $keyword) {
            $recordXML->addChild('keywords', $keyword);
        }

        foreach ($record->getNotes() as $note) {
            $noteXML = $recordXML->addChild('notes');
            $noteXML->addChild('author', $note->getAuthor());
            $noteXML->addChild('date', $note->getDate());
            $noteXML->addChild('path', $note->getPath());
            $noteXML->addChild('text', $note->getText());
        }
        foreach ($record->getSubscriptors() as $subscriptor) {
            $recordXML->addChild('subscriptors', $subscriptor);
        }
        $client->body($recordXML->asXML());
        $response = $this->getClient($client);
        return $this->phpRecord($response->body);
    }

    public function getRecordProperties($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_GET_PROPERTIES);
        $uri .= '?recId=' . $recId;
        $client = Request::get($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return $this->phpRecord($response->body);
    }

    public function deleteRecord($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_DELETE);
        $uri .= '?recId=' . $recId;
        $client = Request::delete($uri);
        $this->getClient($client);
    }

    public function purgeRecord($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_PURGE);
        $uri .= '?recId=' . $recId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function renameRecord($recId, $newName) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_RENAME);
        $uri .= '?recId=' . $recId . '&newName=' . $newName;
        $client = Request::put($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return $this->phpRecord($response->body);
    }

    public function moveRecord($recId, $dstId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_MOVE);
        $uri .= '?recId=' . $recId . '&dstId=' . $dstId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function copyRecord($recId, $dstId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_COPY);
        $uri .= '?recId=' . $recId . '&dstId=' . $dstId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function isValidRecord($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_IS_VALID);
        $uri .= '?recId=' . $recId;
        $client = Request::get($uri);
        $response = $this->getClientWithHTMLResponse($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return (string) $response->body;
    }

    public function getRecordChildren($fldId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_GET_CHILDREN);
        $uri .= '?fldId=' . $fldId;
        $client = Request::get($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        $records = array();
        foreach ($response->body->record as $recordXML) {
            $records[] = $this->phpRecord($recordXML);
        }
        return $records;
    }

    public function lockRecord($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_LOCK);
        $uri .= '?recId=' . $recId;
        $client = Request::put($uri);
        $response = $this->getClient($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return $this->phpLockInfo($response->body);
    }

    public function unlockRecord($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_UNLOCK);
        $uri .= '?recId=' . $recId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function forceUnlockRecord($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_FORCE_UNLOCK);
        $uri .= '?recId=' . $recId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function setRecordTitle($recId) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_SET_TITLE);
        $uri .= '?recId=' . $recId;
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function getRecordPath($uuid) {
        $uri = UriHelper::getUri($this->host, UriHelper::RECORD_GET_PATH);
        $uri .= '/' . $recId;
        $client = Request::put($uri);
        $response = $this->getClientWithHTMLResponse($client);
        $position = strpos($response->body, ':');
        if ($position) {
            $error = substr($response->body, 0, $position);
            $msg = substr($response->body, $position + 1, strlen($response->body));
            if ($error == 'PathNotFoundException') {
                throw new PathNotFoundException($msg);
            }
        }
        return (string) $response->body;
    }

    public function phpRecord($recordXML) {
        $record = new Record();
        $record->setAuthor((string) $recordXML->author);
        $record->setCreated((string) $recordXML->created);
        $record->setNodeClass((int) $recordXML->nodeClass);
        $record->setPath((string) $recordXML->path);
        $record->setPermissions((int) $recordXML->permissions);
        $record->setSubscribed((string) $recordXML->subscribed);
        $record->setUuid((string) $recordXML->uuid);

        //LockInfo        
        $record->setLockInfo($this->phpLockInfo($recordXML->lockInfo));
        $record->setLocked((string) $recordXML->locked);
        $record->setTitle((string) $recordXML->title);

        //categories
        $categories = array();
        foreach ($recordXML->categories as $categoryXML) {
            $categories[] = $this->phpFolder($categoryXML);
        }
        $record->setCategories($categories);
        //keywords
        $keywords = array();
        foreach ($recordXML->keywords as $keywordXML) {
            $keywords[] = (string) $keywordXML;
        }
        $record->setKeywords($keywords);
        //notes
        $notes = array();
        foreach ($recordXML->notes as $noteXML) {
            $note = new Note();
            $note->setAuthor((string) $noteXML->author);
            $note->setDate((string) $noteXML->date);
            $note->setPath((string) $noteXML->path);
            $note->setText((string) $noteXML->text);
            $notes[] = $note;
        }
        $record->setNotes($notes);
        $subscriptors = array();
        foreach ($recordXML->subscriptors as $subscriptor) {
            $subscriptors[] = $subscriptor;
        }
        $record->setSubscriptors($subscriptors);
        return $record;
    }

    public function phpLockInfo($lockInfoXML) {
        $lockInfo = new LockInfo();
        $lockInfo->setNodePath((string) $lockInfoXML->nodePath);
        $lockInfo->setOwner((string) $lockInfoXML->owner);
        $lockInfo->setToken((string) $lockInfoXML->token);
        return $lockInfo;
    }

    public function phpFolder($folderXML) {
        $folder = new Folder();
        $folder->setAuthor((string) $folderXML->author);
        $folder->setCreated((string) $folderXML->created);
        $folder->setNodeClass((int) $folderXML->nodeClass);
        $folder->setPath((string) $folderXML->path);
        $folder->setPermissions((int) $folderXML->permissions);
        $folder->setSubscribed((string) $folderXML->subscribed);
        $folder->setUuid((string) $folderXML->uuid);
        $folder->setHasChildren((string) $folderXML->hasChildren);
        $folder->setStyle((int) $folderXML->style);
        return $folder;
    }

}
