<?php

/**
 * OpenKM, Open Document Management System (http://www.openkm.com)
 * Copyright (c) 2006-2014 Paco Avila & Josep Llort
 * 
 * No bytes were intentionally harmed during the development of this application.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

namespace openkm\impl;

use openkm\bean\AppVersion;
use openkm\bean\Folder;
use openkm\bean\Note;
use openkm\definition\BaseRepository;
use openkm\util\UriHelper;
use Httpful\Request;
use Httpful\Mime;
use openkm\bean\SqlQueryResults;
use openkm\bean\SqlQueryResultColumns;
use openkm\util\Status;
use openkm\util\TypeException;
use openkm\exception\AccessDeniedException;
use openkm\exception\PathNotFoundException;
use openkm\exception\RepositoryException;
use openkm\exception\DatabaseException;
use Httpful\Exception\ConnectionErrorException;
use openkm\exception\UnknowException;
use openkm\exception\ItemExistsException;
use openkm\exception\ExtensionException;
use openkm\exception\AutomationException;
use openkm\exception\LockException;
use openkm\exception\UserQuotaExceededException;
use openkm\exception\IOException;
use openkm\exception\VersionException;

/**
 * RepositoryImpl
 *
 * @author sochoa
 */
class RepositoryImpl implements BaseRepository {

    private $host;
    private $user;
    private $password;

    /**
     * Construct
     */
    public function __construct($host, $user, $password) {
        $this->host = $host;
        $this->user = $user;
        $this->password = $password;
    }

    /**
     * getClient
     */
    private function getClient(Request $client) {
        $client->sendsXml();
        $client->expectsXml();
        $client->authenticateWith($this->user, $this->password);
        return $client->send();
    }

    /**
     * getClientWithHTMLResponse     
     */
    private function getClientWithHTMLResponse(Request $client) {
        $client->sendsXml();
        $client->expectsHtml();
        $client->authenticateWith($this->user, $this->password);
        return $client->send();
    }

    /**
     * Get Root Folder
     * @return \openkm\bean\Folder Return the object Folder of node "/okm:root"
     * @throws ConnectionErrorException when unable to parse or communicate with the server
     * @throws AccessDeniedException
     * @throws PathNotFoundException
     * @throws RepositoryException
     * @throws DatabaseException
     * @throws UnknowException
     */
    public function getRootFolder() {
        try {
            $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_FOLDER);
            $client = Request::get($uri);
            $response = $this->getClient($client);
            if ($response->code == Status::OK) {
                return $this->phpFolderComplete($response->body);
            } else {
                if ($response->code == Status::UNAUTHORIZED) {
                    throw new AccessDeniedException($response->body, $response->code);
                } else if ($response->code == Status::INTERNAL_SERVER_ERROR) {
                    $position = strpos($response->body, ':');
                    if ($position) {
                        $error = substr($response->body, 0, $position);
                        $msg = substr($response->body, $position + 1, strlen($response->body));
                        if ($error == TypeException::PATH_NOT_FOUND_EXCEPTION) {
                            throw new PathNotFoundException($error . ': ' . $msg);
                        } else if ($error == TypeException::REPOSITORY_EXCEPTION) {
                            throw new RepositoryException($error . ': ' . $msg);
                        } else if ($error == TypeException::DATABASE_EXCEPTION) {
                            throw new DatabaseException($error . ': ' . $msg);
                        } else {
                            throw new UnknowException($error . ': ' . $msg);
                        }
                    } else {
                        throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                    }
                } else {
                    throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                }
            }
        } catch (ConnectionErrorException $cee) {
            throw new ConnectionErrorException('ConnectionErrorException: ' . $cee->getMessage());
        }
    }

    /**
     * Get Trash Folder
     * The returned folder will be the user trash folder.
     * For example if the method is executed by "okmAdmin" user then the folder returned will be "/okm:trash/okmAdmin".
     * 
     * @return \openkm\bean\Folder Return the object Folder of node "/okm:trash/{userId}"
     * @throws ConnectionErrorException when unable to parse or communicate with the server
     * @throws AccessDeniedException
     * @throws PathNotFoundException
     * @throws RepositoryException
     * @throws DatabaseException
     * @throws UnknowException
     */
    public function getTrashFolder() {
        try {
            $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_TRASH);
            $client = Request::get($uri);
            $response = $this->getClient($client);
            if ($response->code == Status::OK) {
                return $this->phpFolderComplete($response->body);
            } else {
                if ($response->code == Status::UNAUTHORIZED) {
                    throw new AccessDeniedException($response->body, $response->code);
                } else if ($response->code == Status::INTERNAL_SERVER_ERROR) {
                    $position = strpos($response->body, ':');
                    if ($position) {
                        $error = substr($response->body, 0, $position);
                        $msg = substr($response->body, $position + 1, strlen($response->body));
                        if ($error == TypeException::PATH_NOT_FOUND_EXCEPTION) {
                            throw new PathNotFoundException($error . ': ' . $msg);
                        } else if ($error == TypeException::REPOSITORY_EXCEPTION) {
                            throw new RepositoryException($error . ': ' . $msg);
                        } else if ($error == TypeException::DATABASE_EXCEPTION) {
                            throw new DatabaseException($error . ': ' . $msg);
                        } else {
                            throw new UnknowException($error . ': ' . $msg);
                        }
                    } else {
                        throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                    }
                } else {
                    throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                }
            }
        } catch (ConnectionErrorException $cee) {
            throw new ConnectionErrorException('ConnectionErrorException: ' . $cee->getMessage());
        }
    }

    /**
     * Get Trash Folder Base
     * @return \openkm\bean\Folder Return the object Folder of node "/okm:trash"
     * @throws ConnectionErrorException when unable to parse or communicate with the server
     * @throws AccessDeniedException
     * @throws PathNotFoundException
     * @throws RepositoryException
     * @throws DatabaseException
     * @throws UnknowException
     */
    public function getTrashFolderBase() {
        try {
            $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_TRASH_BASE);
            $client = Request::get($uri);
            $response = $this->getClient($client);
            if ($response->code == Status::OK) {
                return $this->phpFolderComplete($response->body);
            } else {
                if ($response->code == Status::UNAUTHORIZED) {
                    throw new AccessDeniedException($response->body, $response->code);
                } else if ($response->code == Status::INTERNAL_SERVER_ERROR) {
                    $position = strpos($response->body, ':');
                    if ($position) {
                        $error = substr($response->body, 0, $position);
                        $msg = substr($response->body, $position + 1, strlen($response->body));
                        if ($error == TypeException::PATH_NOT_FOUND_EXCEPTION) {
                            throw new PathNotFoundException($error . ': ' . $msg);
                        } else if ($error == TypeException::REPOSITORY_EXCEPTION) {
                            throw new RepositoryException($error . ': ' . $msg);
                        } else if ($error == TypeException::DATABASE_EXCEPTION) {
                            throw new DatabaseException($error . ': ' . $msg);
                        } else {
                            throw new UnknowException($error . ': ' . $msg);
                        }
                    } else {
                        throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                    }
                } else {
                    throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                }
            }
        } catch (ConnectionErrorException $cee) {
            throw new ConnectionErrorException('ConnectionErrorException: ' . $cee->getMessage());
        }
    }

    /**
     * Get Templates Folder
     * @return \openkm\bean\Folder Return the object Folder of node "/okm:templates"
     * @throws ConnectionErrorException when unable to parse or communicate with the server
     * @throws AccessDeniedException
     * @throws PathNotFoundException
     * @throws RepositoryException
     * @throws DatabaseException
     * @throws UnknowException
     */
    public function getTemplatesFolder() {
        try {
            $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_TEMPLATES);
            $client = Request::get($uri);
            $response = $this->getClient($client);
            if ($response->code == Status::OK) {
                return $this->phpFolderComplete($response->body);
            } else {
                if ($response->code == Status::UNAUTHORIZED) {
                    throw new AccessDeniedException($response->body, $response->code);
                } else if ($response->code == Status::INTERNAL_SERVER_ERROR) {
                    $position = strpos($response->body, ':');
                    if ($position) {
                        $error = substr($response->body, 0, $position);
                        $msg = substr($response->body, $position + 1, strlen($response->body));
                        if ($error == TypeException::PATH_NOT_FOUND_EXCEPTION) {
                            throw new PathNotFoundException($error . ': ' . $msg);
                        } else if ($error == TypeException::REPOSITORY_EXCEPTION) {
                            throw new RepositoryException($error . ': ' . $msg);
                        } else if ($error == TypeException::DATABASE_EXCEPTION) {
                            throw new DatabaseException($error . ': ' . $msg);
                        } else {
                            throw new UnknowException($error . ': ' . $msg);
                        }
                    } else {
                        throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                    }
                } else {
                    throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                }
            }
        } catch (ConnectionErrorException $cee) {
            throw new ConnectionErrorException('ConnectionErrorException: ' . $cee->getMessage());
        }
    }

    /**
     * Get Personal Folder
     * The returned folder will be the user personal folder.
     * For example if the method is executed by "okmAdmin" user then the folder returned will be "/okm:personal/okmAdmin".
     * 
     * @return \openkm\bean\Folder Return the object Folder of node "/okm:personal/{userId}"
     * @throws ConnectionErrorException when unable to parse or communicate with the server
     * @throws AccessDeniedException
     * @throws PathNotFoundException
     * @throws RepositoryException
     * @throws DatabaseException
     * @throws UnknowException
     */
    public function getPersonalFolder() {
        try {
            $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_PERSONAL);
            $client = Request::get($uri);
            $response = $this->getClient($client);
            if ($response->code == Status::OK) {
                return $this->phpFolderComplete($response->body);
            } else {
                if ($response->code == Status::UNAUTHORIZED) {
                    throw new AccessDeniedException($response->body, $response->code);
                } else if ($response->code == Status::INTERNAL_SERVER_ERROR) {
                    $position = strpos($response->body, ':');
                    if ($position) {
                        $error = substr($response->body, 0, $position);
                        $msg = substr($response->body, $position + 1, strlen($response->body));
                        if ($error == TypeException::PATH_NOT_FOUND_EXCEPTION) {
                            throw new PathNotFoundException($error . ': ' . $msg);
                        } else if ($error == TypeException::REPOSITORY_EXCEPTION) {
                            throw new RepositoryException($error . ': ' . $msg);
                        } else if ($error == TypeException::DATABASE_EXCEPTION) {
                            throw new DatabaseException($error . ': ' . $msg);
                        } else {
                            throw new UnknowException($error . ': ' . $msg);
                        }
                    } else {
                        throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                    }
                } else {
                    throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                }
            }
        } catch (ConnectionErrorException $cee) {
            throw new ConnectionErrorException('ConnectionErrorException: ' . $cee->getMessage());
        }
    }

    /**
     * Get Personal Folder Base
     * @return \openkm\bean\Folder Return the object Folder of node "/okm:personal"
     * @throws ConnectionErrorException when unable to parse or communicate with the server
     * @throws AccessDeniedException
     * @throws PathNotFoundException
     * @throws RepositoryException
     * @throws DatabaseException
     * @throws UnknowException
     */
    public function getPersonalFolderBase() {
        try {
            $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_PERSONAL_BASE);
            $client = Request::get($uri);
            $response = $this->getClient($client);
            if ($response->code == Status::OK) {
                return $this->phpFolderComplete($response->body);
            } else {
                if ($response->code == Status::UNAUTHORIZED) {
                    throw new AccessDeniedException($response->body, $response->code);
                } else if ($response->code == Status::INTERNAL_SERVER_ERROR) {
                    $position = strpos($response->body, ':');
                    if ($position) {
                        $error = substr($response->body, 0, $position);
                        $msg = substr($response->body, $position + 1, strlen($response->body));
                        if ($error == TypeException::PATH_NOT_FOUND_EXCEPTION) {
                            throw new PathNotFoundException($error . ': ' . $msg);
                        } else if ($error == TypeException::REPOSITORY_EXCEPTION) {
                            throw new RepositoryException($error . ': ' . $msg);
                        } else if ($error == TypeException::DATABASE_EXCEPTION) {
                            throw new DatabaseException($error . ': ' . $msg);
                        } else {
                            throw new UnknowException($error . ': ' . $msg);
                        }
                    } else {
                        throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                    }
                } else {
                    throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                }
            }
        } catch (ConnectionErrorException $cee) {
            throw new ConnectionErrorException('ConnectionErrorException: ' . $cee->getMessage());
        }
    }

    public function getMailFolder() {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_MAIL);
        $client = Request::get($uri);
        $response = $this->getClient($client);
        return $this->phpFolderComplete($response->body);
    }

    public function getMailFolderBase() {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_MAIL_BASE);
        $client = Request::get($uri);
        $response = $this->getClient($client);
        return $this->phpFolderComplete($response->body);
    }

    public function getThesaurusFolder() {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_THESAURUS);
        $client = Request::get($uri);
        $response = $this->getClient($client);
        return $this->phpFolderComplete($response->body);
    }

    public function getCategoriesFolder() {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_ROOT_CATEGORIES);
        $client = Request::get($uri);
        $response = $this->getClient($client);
        return $this->phpFolderComplete($response->body);
    }

    public function purgeTrash() {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_PURGE_TRASH);
        $client = Request::delete($uri);
        $response = $this->getClient($client);
    }

    public function getUpdateMessage() {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_UPDATE_MESSAGE);
        $client = Request::get($uri);
        $response = $this->getClientWithHTMLResponse($client);
        return $response->body;
    }

    public function getRepositoryUuid() {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_RESPOSITORY_UUID);
        $client = Request::get($uri);
        $response = $this->getClientWithHTMLResponse($client);
        return $response->body;
    }

    public function hasNode($nodeId) {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_HAS_NODE);
        $uri .= '?nodeId=' . $nodeId;
        $client = Request::get($uri);
        $response = $this->getClientWithHTMLResponse($client);
        return $response->body;
    }

    public function getNodePath($uuid) {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_NODE_PATH);
        $uri .= '?uuid=' . $uuid;
        $client = Request::get($uri);
        $response = $this->getClientWithHTMLResponse($client);
        return $response->body;
    }

    public function getNodeUuid($nodePath) {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_NODE_UUID);
        $uri .= '?nodePath=' . $nodePath;
        $client = Request::get($uri);
        $response = $this->getClientWithHTMLResponse($client);
        return (string) $response->body;
    }

    /**
     * Get App Version
     * @return AppVersion Return information about OpenKM version.
     * @throws ConnectionErrorException
     * @throws AccessDeniedException
     * @throws PathNotFoundException
     * @throws RepositoryException
     * @throws DatabaseException
     * @throws UnknowException
     */
    public function getAppVersion() {
        try {
            $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_GET_APP_VERSION);
            $client = Request::get($uri);
            $response = $this->getClient($client);
            if ($response->code == Status::OK) {
                $appVersionXML = $response->body;
                $appVersion = new AppVersion();
                $appVersion->setBuild((string) $appVersionXML->build);
                $appVersion->setExtension((string) $appVersionXML->estension);
                $appVersion->setMaintenance((string) $appVersionXML->maintenance);
                $appVersion->setMajor((string) $appVersionXML->major);
                $appVersion->setMinor((string) $appVersionXML->minor);
                return $appVersion;
            } else {
                if ($response->code == Status::UNAUTHORIZED) {
                    throw new AccessDeniedException($response->body, $response->code);
                } else if ($response->code == Status::INTERNAL_SERVER_ERROR) {
                    $position = strpos($response->body, ':');
                    if ($position) {
                        $error = substr($response->body, 0, $position);
                        $msg = substr($response->body, $position + 1, strlen($response->body));
                        if ($error == TypeException::PATH_NOT_FOUND_EXCEPTION) {
                            throw new PathNotFoundException($error . ': ' . $msg);
                        } else if ($error == TypeException::REPOSITORY_EXCEPTION) {
                            throw new RepositoryException($error . ': ' . $msg);
                        } else if ($error == TypeException::DATABASE_EXCEPTION) {
                            throw new DatabaseException($error . ': ' . $msg);
                        } else {
                            throw new UnknowException($error . ': ' . $msg);
                        }
                    } else {
                        throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                    }
                } else {
                    throw new UnknowException("HTTP error code " + $response->code + ": " + $response->body);
                }
            }
        } catch (ConnectionErrorException $cee) {
            throw new ConnectionErrorException('ConnectionErrorException: ' . $cee->getMessage());
        }
    }

    public function copyAttributes($nodeId, $dstId, $categories, $keywords, $propertyGroups, $notes, $wiki) {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_COPY_ATTRIBUTES);
        $uri .= '?nodeId=' . $nodeId . '&dstId=' . $dstId;
        if ($categories == 1) {
            $uri .= '&categories=true';
        } else {
            $uri.= '&categories=false';
        }
        if ($keywords == 1) {
            $uri .= '&keywords=true';
        } else {
            $uri.= '&keywords=false';
        }
        if ($propertyGroups == 1) {
            $uri .= '&propertyGroups=true';
        } else {
            $uri .= '&propertyGroups=false';
        }
        if ($notes == 1) {
            $uri .= '&notes=true';
        } else {
            $uri.= '&notes=false';
        }
        if ($wiki == 1) {
            $uri .= '&wiki=true';
        } else {
            $uri.= '&wiki=false';
        }
        $client = Request::put($uri);
        $this->getClient($client);
    }

    public function executeScript($query) {
        
    }

    public function executeSqlQuery($query) {
        $uri = UriHelper::getUri($this->host, UriHelper::REPOSITORY_EXECUTE_SQL_QUERY);
        $params = array();
        $params['query'] = $query;
        $client = Request::post($uri);
        $client->body($params, Mime::UPLOAD);
        $client->authenticateWith($this->user, $this->password);
        $client->expects(Mime::XML);
        $response = $client->send();
        $sqlQueryResults = new SqlQueryResults();
        $results = array();
        foreach ($response->body->sqlQueryResult as $sqlQueryResultColumnsXML) {
            $sqlQueryResultColumns = new SqlQueryResultColumns();
            $columns = array();
            foreach ($sqlQueryResultColumnsXML->sqlQueryResultColumn as $columnsXML) {
                $columns[] = (string) $columnsXML;
            }
            $sqlQueryResultColumns->setColumns($columns);
            $results[] = $sqlQueryResultColumns;
        }
        $sqlQueryResults->setResults($results);
        return $sqlQueryResults;
    }

    public function executeHqlQuery($query) {
        
    }

    public function getTranslations($lang, $module) {
        
    }

    /**
     * phpFolderComplete
     * @param type $folderXML
     * @return type \openkm\bean\Folder
     */
    public function phpFolderComplete($folderXML) {
        $folder = new Folder();
        $folder = $this->phpFolder($folderXML);
        $categories = array();
        foreach ($folderXML->categories as $categoryXML) {
            $category = new Folder();
            $category = $this->phpFolder($categoryXML);
            $categories[] = $category;
        }
        $folder->setCategories($categories);
        $keywords = array();
        foreach ($folderXML->keywords as $keywordXML) {
            $keywords[] = (string) $keywordXML;
        }
        $folder->setKeywords($keywords);
        $notes = array();
        foreach ($folderXML->notes as $noteXML) {
            $note = new Note();
            $note->setAuthor((string) $noteXML->author);
            $note->setDate((string) $noteXML->date);
            $note->setPath((string) $noteXML->path);
            $note->setText((string) $noteXML->text);
            $notes[] = $note;
        }
        $folder->setNotes($notes);
        return $folder;
    }

    public function phpFolder($folderXML) {
        $folder = new Folder();
        $folder->setAuthor((string) $folderXML->author);
        $folder->setCreated((string) $folderXML->created);
        $folder->setPath((string) $folderXML->path);
        $folder->setPermissions((int) $folderXML->permissions);
        $folder->setSubscribed((string) $folderXML->subscribed);
        $folder->setUuid((string) $folderXML->uuid);
        $folder->setHasChildren((string) $folderXML->hasChildren);
        $folder->setStyle((int) $folderXML->style);
        return $folder;
    }

}

?>
